;;; init.el --- Init de emacs ordenado -*- lexical-binding: t -*-

;; Author: kj <webmaster@outcontrol.net>
;; URL: https://git.kj2.me/kj/confi-emacs-actual

;;; Commentary:

;; Desde aquí se cargan todos los archivos de la carpeta config.

;;; Code:

;; Optimize Garbage Collection for Startup
(setq gc-cons-threshold most-positive-fixnum)

;; Optimize `auto-mode-alist`
(setq auto-mode-case-fold nil)

(unless (or (daemonp) noninteractive init-file-debug)
  ;; Temporarily suppress file-handler processing to speed up startup
  (let ((default-handlers file-name-handler-alist))
    (setq file-name-handler-alist nil)
    ;; Recover handlers after startup
    (add-hook 'emacs-startup-hook
              (lambda ()
                (setq file-name-handler-alist
                      (delete-dups (append file-name-handler-alist default-handlers))))
              101)))

;; Función para añadir 'config' a la ruta de carga código elisp de emacs
(defun update-load-path (&rest _)
  "Update the `load-path` to prioritize personal configurations."
    (push (expand-file-name "configs" user-emacs-directory) load-path))

;; Nos aseguramos de que ejecute 'update-load-path' antes de 'package-initialize'
(advice-add #'package-initialize :after #'update-load-path)

;; Ahora lo corremos implícitamente
(update-load-path)

;; Paquetes base
(require 'init-package)
(require 'init-base)
(require 'init-utils)
(require 'init-ui)
(require 'init-editor)
(require 'init-highlight)
(require 'init-theme)
(require 'init-ai)
(require 'init-vc)
(require 'init-org)
(require 'init-lsp)
(require 'init-completion)
(require 'init-minibuffer)
;; (require 'init-evil)
(require 'init-keys)

;; Lenguajes
(require 'lang-php)
(require 'lang-js)
(require 'lang-go)
(require 'lang-rust)
(require 'lang-html)

(provide 'init)
;;; init.el ends here
